@@ -9,53 +9,46 @@ p_load(Rcpp,
        ggthemes,
        broom,
        miceFast,
-       data.table)
-
+       data.table,
+       microbenchmark,
+       car)
 
 set.seed(1234)
-
 #parameters
-
-power = 6 # power of 10 - number of observations - should be adjusted to a computer capabilities
-
-nr_var = 7 #CHANGE - only if you generate a bigger corr matrix:  number of variables - independent and one dependent
-
+power = 5 # power of 10 - number of observations - should be adjusted to a computer capabilities
+nr_var = 20 #CHANGE - only if you generate a bigger corr matrix:  number of variables - independent and one dependent
 grs = max(c(10**(power-3),10)) # grouping variable - number of groups
-
-iters = 10 # number of iterations for benchmarking
-
+iters = 30 # number of iterations for benchmarking
 ## generete example - data
 
 ##positive-defined correlation matrix
+#cors = matrix(c(1,0.6,0.7,0.4,0.4,0.5,0.35,
+#                NA,1,0.2,0.05,0.1,0.12,0.15,
+#                NA,NA,1,0.15,0.15,0.1,0.08,
+#                NA,NA,NA,1,0.12,0.15,0.1,
+#                NA,NA,NA,NA,1,0.15,0.2,
+#                NA,NA,NA,NA,NA,1,0.15,
+#                NA,NA,NA,NA,NA,NA,1),7,7,byrow = T)
 
-cors = matrix(c(1,0.6,0.7,0.4,0.4,0.5,0.35,
-                NA,1,0.2,0.05,0.1,0.12,0.15,
-                NA,NA,1,0.15,0.15,0.1,0.08,
-                NA,NA,NA,1,0.12,0.15,0.1,
-                NA,NA,NA,NA,1,0.15,0.2,
-                NA,NA,NA,NA,NA,1,0.15,
-                NA,NA,NA,NA,NA,NA,1),7,7,byrow = T)
+#cors[lower.tri(cors)] = t(cors)[lower.tri(cors)]
+# automatic corr matrix
 
-cors[lower.tri(cors)] = t(cors)[lower.tri(cors)]
-
-# automatic corr matrix - close to diagonal
-
-#cors = stats::rWishart(100,nr_var+10,diag(nr_var))
-
-#cors = apply(cors,1:2,mean)/(nr_var+10)
+vars_mat = diag(nr_var)
+vars_mat[vars_mat==0] = 0.3
+covs = stats::rWishart(3,nr_var,vars_mat)
+covs = apply(covs,1:2,mean)/(nr_var+10)
+cors = diag(1/sqrt(diag(covs))) %*% covs%*%diag(1/sqrt(diag(covs)))
 
+#diag(cors) = 1
 #cors
-
 ##
 
 model = new(corrData,10,10^power,rep(0,nr_var),cors)
-
 data_bin = model$fill("binom")
 data_disc = model$fill("discrete")
 data_con = model$fill("contin")
 
 n_vars = ncol(cors)
-
 posit_y = 1
 posit_x = 2:(n_vars-2)
 posit_w = n_vars-1
@@ -65,12 +58,9 @@ posit_index = n_vars+2
 
 ## NA index
 
-index_NA = 1:nrow(data_con) %in% sample(1:nrow(data_con),10^(power-1))
-
+index_NA = 1:nrow(data_con) %in% sample(1:nrow(data_con),2*10^(power-1))
 fill_by_NA = function(v,index_NA){
-
   v[index_NA] = NA
-
   v
 }
 
@@ -103,22 +93,24 @@ model = new(miceFast)
 data = data_disc_NA[,c(posit_y,posit_x)]
 model$set_data(data)
 pred_miceFast =  model$impute("lda",posit_y,posit_x)
+rm(model)
 
 table(pred_miceFast$imputations[index_NA] ,data_disc[index_NA,posit_y])
 table(as.numeric(mice.impute.lda),data_disc[index_NA,posit_y])
 
-m1 = microbenchmark::microbenchmark(R=mice.impute.lda(data_disc[,posit_y],!index_NA,data_disc[,posit_x]),
+m1 = microbenchmark::microbenchmark(R_mice=mice.impute.lda(data_disc[,posit_y],!index_NA,data_disc[,posit_x]),
                                miceFast={
                                  model = new(miceFast)
                                  model$set_data(data_disc_NA[,c(posit_y,posit_x)])
                                  pred_miceFast =  model$impute("lda",posit_y,posit_x)
+                                 rm(model)
                                },
                                times=iters)
 m1
 
 g1 = autoplot(m1,log=FALSE)+theme_economist()+ ggtitle("LDA discrete - without grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata/images/g1.png",g1)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g1.png",g1)
 
 ### grouping variable
 
@@ -150,11 +142,11 @@ pred_dplyr = data_disc_NA_sort_DF %>%
 table(pred_dplyr,true_y)
 
 data_disc_NA_sort_DT = data.table(data_disc_NA_sort)
-pred_datatable = data_disc_NA_sort_DT[,{im=mice.impute.lda(as.matrix(.SD[,1]),!index_NA,as.matrix(.SD[,c(2,3,4,5)]))},by=.(group)]
+pred_datatable = data_disc_NA_sort_DT[,{im=mice.impute.lda(as.matrix(.SD[['y']]),!index_NA,as.matrix(.SD[,posit_x,with=F]))},by=.(group)]
 
 table(pred_datatable[['V1']],true_y)
 
-pred_datatable_miceFast = data_disc_NA_sort_DT[,{im=fill_NA(as.matrix(.SD),"lda",1,c(2,3,4,5))},by=.(group)]
+pred_datatable_miceFast = data_disc_NA_sort_DT[,{im=fill_NA(as.matrix(.SD),"lda",posit_y,posit_x)},by=.(group)]
 
 table(pred_datatable_miceFast[['V1']][index_NA[index_sort]],true_y)
 
@@ -165,6 +157,7 @@ model = new(miceFast)
 model$set_data(data)
 model$set_g(g)
 pred_miceFast =  model$impute("lda",posit_y,posit_x)
+rm(model)
 
 table(pred_miceFast$imputations[as.logical(pred_miceFast$index_imp)],true_y)
 
@@ -180,7 +173,7 @@ table(pred_miceFast_rotate$imputations[order(model$get_index())][as.logical(pred
 ##Performance
 
 m2 = microbenchmark::microbenchmark(
-  dplyr={
+  dplyr_mice={
   pred_dplyr = data_disc_NA_sort_DF %>%
     group_by(group) %>%
     do(im = mice.impute.lda(as.matrix(.[,posit_y]),!.$index_NA,as.matrix(.[,posit_x]))) %>%
@@ -188,8 +181,9 @@ m2 = microbenchmark::microbenchmark(
     ungroup()%>%
     select(x) %>%
     unlist() },
-  DT={pred_datatable= data_disc_NA_sort_DT[,{im=mice.impute.lda(as.matrix(.SD[,1]),!index_NA,as.matrix(.SD[,c(2,3,4,5)]))},by=.(group)]},
-  R_base={
+  DT_mice={pred_datatable= data_disc_NA_sort_DT[,{im=mice.impute.lda(as.matrix(.SD[['y']]),!index_NA,as.matrix(.SD[,posit_x,with=F]))},by=.(group)]},
+  DT_miceFast = {pred_datatable_miceFast = data_disc_NA_sort_DT[,{im=fill_NA(as.matrix(.SD),"lda",posit_y,posit_x)},by=.(group)]},
+  R_mice={
       pred_all = NULL
       for(i in unique(data_disc_NA_sort[,nr_var])){
         sub = data_disc_NA_sort[,posit_grs]==i
@@ -201,6 +195,7 @@ m2 = microbenchmark::microbenchmark(
     model$set_data(data)
     model$set_g(g)
     pred_miceFast =  model$impute("lda",posit_y,posit_x)
+    rm(model)
     },
   times=iters)
 
@@ -208,7 +203,7 @@ m2
 
 g2 = autoplot(m2,log=FALSE)+theme_economist()+ ggtitle("LDA discrete - with grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g2.png",g2)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g2.png",g2)
 
 
 #######################Binom
@@ -220,15 +215,18 @@ data = data_bin_NA[,c(posit_y,posit_x)]
 model = new(miceFast)
 model$set_data(data)
 pred_miceFast =  model$impute("lda",posit_y,posit_x)
+rm(model)
 
 table(pred_miceFast$imputations[index_NA] ,data_bin[index_NA,posit_y])
 table(mice.impute.lda,data_bin[index_NA,posit_y])
 
-m3 = microbenchmark::microbenchmark(R={mice.impute.lda = mice.impute.lda(data_bin[,posit_y],!index_NA,data_bin[,posit_x])},
+m3 = microbenchmark::microbenchmark(R_mice={mice.impute.lda = mice.impute.lda(data_bin[,posit_y],!index_NA,data_bin[,posit_x])},
                                miceFast={
                                  model = new(miceFast)
                                  model$set_data(data)
                                  pred_miceFast =  model$impute("lda",posit_y,posit_x)
+                                 rm(model)
+
                                },
                                times=iters)
 
@@ -236,7 +234,7 @@ m3
 
 g3 = autoplot(m3,log=FALSE)+theme_economist()+ ggtitle("LDA binom - without grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g3.png",g3)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g3.png",g3)
 
 
 #####################Continous - LM Noise
@@ -249,28 +247,28 @@ data = data_con_NA[,c(posit_y,posit_x)]
 model = new(miceFast)
 model$set_data(data)
 pred_miceFast =  model$impute("lm_noise",posit_y,posit_x)
+rm(model)
 
 sum((pred_miceFast$imputations[index_NA] - data_con[index_NA,posit_y])^2)
 sum((mice.impute.norm.nob - data_con[index_NA,posit_y])^2)
 
-m4 = microbenchmark::microbenchmark(R ={mice.impute.norm.nob = mice.impute.norm.nob(data_con[,posit_y],!index_NA,data_con[,posit_x])},
+m4 = microbenchmark::microbenchmark(R_mice ={mice.impute.norm.nob = mice.impute.norm.nob(data_con[,posit_y],!index_NA,data_con[,posit_x])},
                                miceFast={
                                  model = new(miceFast)
                                  model$set_data(data)
                                  pred_miceFast =  model$impute("lm_noise",posit_y,posit_x)
+                                 rm(model)
+
                                },
                                times=iters)
 m4
 
 g4 = autoplot(m4,log=FALSE)+theme_economist()+ ggtitle("linear regression noise - without grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g4.png",g4)
-
-
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g4.png",g4)
 
 #####################Continous - LM Bayes
 
-
 mice.impute.norm.bayes = mice.impute.norm(data_con[,posit_y],!index_NA,data_con[,posit_x])
 
 data = data_con_NA[,c(posit_y,posit_x)]
@@ -278,22 +276,25 @@ data = data_con_NA[,c(posit_y,posit_x)]
 model = new(miceFast)
 model$set_data(data)
 pred_miceFast =  model$impute("lm_bayes",posit_y,posit_x)
+rm(model)
 
 sum((pred_miceFast$imputations[index_NA] - data_con[index_NA,posit_y])^2)
 sum((mice.impute.norm.bayes - data_con[index_NA,posit_y])^2)
 
-m5 = microbenchmark::microbenchmark(R = mice.impute.norm(data_con[,posit_y],!index_NA,data_con[,posit_x]),
+m5 = microbenchmark::microbenchmark(R_mice = mice.impute.norm(data_con[,posit_y],!index_NA,data_con[,posit_x]),
                                miceFast={
                                  model = new(miceFast)
                                  model$set_data(data)
                                  pred_miceFast =  model$impute("lm_bayes",posit_y,posit_x)
+                                 rm(model)
+
                                },
                                times=iters)
 m5
 
 g5 = autoplot(m5,log=FALSE)+theme_economist()+ ggtitle("linear regression bayes - without grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g5.png",g5)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g5.png",g5)
 
 #####################Continous - LM Predict
 
@@ -305,24 +306,27 @@ data = cbind(data_con_NA[,c(posit_y,posit_x)],1)
 model = new(miceFast)
 model$set_data(data)
 pred_miceFast =  model$impute("lm_pred",posit_y,c(posit_x,max(posit_x)+1))
+rm(model)
 
 sum((pred_miceFast$imputations[index_NA] - data_con[index_NA,posit_y])^2)
 sum((mice.impute.norm.pred - data_con[index_NA,posit_y])^2)
 
-m6 = microbenchmark::microbenchmark(R = {
+m6 = microbenchmark::microbenchmark(R_mice = {
   mice.impute.norm.pred = mice.impute.norm.predict(data_con[,posit_y],!index_NA,data_con[,posit_x])
 },
 miceFast={
   model = new(miceFast)
   model$set_data(data)
   pred_miceFast =  model$impute("lm_pred",posit_y,posit_x)
+  rm(model)
+
 },times=iters)
 
 m6
 
 g6 = autoplot(m6,log=FALSE)+theme_economist()+ ggtitle("linear regression predict - without grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g6.png",g6)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g6.png",g6)
 
 
 ## grouping variable
@@ -349,9 +353,9 @@ pred_dplyr = data_con_NA_sort %>%
 
 data_con_NA_sort_DT = data.table(data_con_NA_sort)
 
-pred_datatable = data_con_NA_sort_DT[,{im=mice.impute.norm.predict(as.matrix(.SD[,1]),!index_NA,as.matrix(.SD[,c(2,3,4,5)]))},by=.(group)]
+pred_datatable = data_con_NA_sort_DT[,{im=mice.impute.norm.predict(as.matrix(.SD[,1]),!index_NA,as.matrix(.SD[,posit_x,with=F]))},by=.(group)]
 
-pred_datatable_miceFast = data_con_NA_sort_DT[,{im=fill_NA(cbind(as.matrix(.SD),1),"lm_pred",1,c(2,3,4,5,9))},by=.(group)]
+pred_datatable_miceFast = data_con_NA_sort_DT[,{im=fill_NA(cbind(as.matrix(.SD),1),"lm_pred",1,posit_x)},by=.(group)]
 
 data = cbind(data_con_NA_sort[,c(posit_y,posit_x)],1)
 g = data_con_NA_sort[,posit_grs]
@@ -359,20 +363,21 @@ g = data_con_NA_sort[,posit_grs]
 model = new(miceFast)
 model$set_data(data)
 model$set_g(g)
-pred_miceFast =  model$impute("lm_pred",posit_y,c(posit_x,max(posit_x)+1))
+pred_miceFast =  model$impute("lm_pred",posit_y,posit_x)
+rm(model)
 
 true_y = data_con[index_sort,][index_NA[index_sort],posit_y]
 
-sum((pred_miceFast$imputations[as.logical(pred_miceFast$index_imputed)]-true_y)^2)
+sum((pred_miceFast$imputations[as.logical(pred_miceFast$index_imputed)]-true_y)^2) # no intercept
 sum((pred_dplyr-true_y)^2)
 sum((pred_Rbase-true_y)^2)
 sum((pred_datatable$V1-true_y)^2)
-sum((pred_datatable_miceFast$V1[index_NA[index_sort]]-true_y)^2)
+sum((pred_datatable_miceFast$V1[index_NA[index_sort]]-true_y)^2) #no intercept
 
 ##Performance
 
 m7 = microbenchmark::microbenchmark(
-  dplyr={
+  dplyr_mice={
     pred_dplyr = data_con_NA_sort %>%
       as.data.frame() %>%
       group_by(group) %>%
@@ -383,7 +388,7 @@ m7 = microbenchmark::microbenchmark(
       unlist() %>%
       as.numeric()
     },
-  R_base={
+  R_mice={
     pred_Rbase = NULL
     for(i in unique(data_con_NA_sort[,posit_grs])){
       sub = data_con_NA_sort[,posit_grs]==i
@@ -396,9 +401,12 @@ m7 = microbenchmark::microbenchmark(
     model = new(miceFast)
     model$set_data(data)
     model$set_g(g)
-    pred_miceFast =  model$impute("lm_pred",posit_y,posit_x)},
+    pred_miceFast =  model$impute("lm_pred",posit_y,posit_x)
+    rm(model)
+  },
   times=iters,
-  DT={pred_datatable= data_disc_NA_sort_DT[,{im=mice.impute.norm.predict(as.matrix(.SD[,1]),!index_NA,as.matrix(.SD[,c(2,3,4,5)]))},by=.(group)]}
+  DT_mice={pred_datatable= data_disc_NA_sort_DT[,{im=mice.impute.norm.predict(as.matrix(.SD[,1]),!index_NA,as.matrix(.SD[,posit_x,with=F]))},by=.(group)]},
+  DT_miceFast = {pred_datatable_DT=data_disc_NA_sort_DT[,{im=fill_NA(as.matrix(.SD),"lm_pred",posit_y,posit_x)},by=.(group)]}
 )
 
 m7
@@ -407,15 +415,12 @@ g7 = autoplot(m7,log=FALSE)+
   theme_economist()+
   ggtitle("linear regression predict - with grouping")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g7.png",g7)
-
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g7.png",g7)
 
 ####
 ####Multiple Imputations
 ####
 
-
-
 mice.impute.norm.nob = rowMeans(sapply(1:10,function(x) mice.impute.norm.nob(data_con[,posit_y],!index_NA,data_con[,posit_x])))
 
 data = data_con_NA[,c(posit_y,posit_x)]
@@ -423,24 +428,27 @@ data = data_con_NA[,c(posit_y,posit_x)]
 model = new(miceFast)
 model$set_data(data)
 pred_miceFast =  model$impute_N("lm_noise",posit_y,posit_x,10)
+rm(model)
 
 sum((pred_miceFast$imputations[index_NA] - data_con[index_NA,posit_y])^2)
 sum((mice.impute.norm.nob - data_con[index_NA,posit_y])^2)
 
-m8 = microbenchmark::microbenchmark(R ={mice.impute.norm.nob = rowMeans(sapply(1:10,function(x) mice.impute.norm.nob(data_con[,posit_y],!index_NA,data_con[,posit_x])))},
+m8 = microbenchmark::microbenchmark(R_mice ={mice.impute.norm.nob = rowMeans(sapply(1:10,function(x) mice.impute.norm.nob(data_con[,posit_y],!index_NA,data_con[,posit_x])))},
                                     miceFast={
                                       model = new(miceFast)
                                       model$set_data(data)
                                       pred_miceFast =  model$impute_N("lm_noise",posit_y,posit_x,10)
+                                      rm(model)
+
                                     },
                                     times=iters)
 m8
 
 g8 = autoplot(m8,log=FALSE)+theme_economist()+ ggtitle("linear regression noise - without grouping - multiple 10")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g8.png",g8)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g8.png",g8)
 
-#####################Continous - LM Bayes
+#####################Continous - LM Bayes - multiple
 
 mice.impute.norm.bayes = rowMeans(sapply(1:10,function(x) mice.impute.norm(data_con[,posit_y],!index_NA,data_con[,posit_x])))
 
@@ -449,23 +457,54 @@ data = data_con_NA[,c(posit_y,posit_x)]
 model = new(miceFast)
 model$set_data(data)
 pred_miceFast =  model$impute_N("lm_bayes",posit_y,posit_x,10)
+rm(model)
 
 sum((pred_miceFast$imputations[index_NA] - data_con[index_NA,posit_y])^2)
 sum((mice.impute.norm.bayes - data_con[index_NA,posit_y])^2)
 
-m9 = microbenchmark::microbenchmark(R = {mice.impute.norm.bayes = rowMeans(sapply(1:10,function(x) mice.impute.norm(data_con[,posit_y],!index_NA,data_con[,posit_x])))},
+m9 = microbenchmark::microbenchmark(R_mice = {mice.impute.norm.bayes = rowMeans(sapply(1:10,function(x) mice.impute.norm(data_con[,posit_y],!index_NA,data_con[,posit_x])))},
                                     miceFast={
                                       model = new(miceFast)
                                       model$set_data(data)
                                       pred_miceFast =  model$impute_N("lm_bayes",posit_y,posit_x,10)
+                                      rm(model)
+
                                     },
                                     times=iters)
 m9
 
 g9 = autoplot(m9,log=FALSE)+theme_economist()+ ggtitle("linear regression bayes - without grouping - multiple 10")
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata//images/g9.png",g9)
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g9.png",g9)
 
+#####################Continous - VIFS
+
+data.vifs = data.frame(data_con[,c(posit_y,posit_x)])
+colnames(data.vifs) = letters[1:(nr_var-2)]
+vifs_car = car::vif(lm(a~.,data=data.vifs))
+
+model = new(miceFast)
+model$set_data(data_con)
+vifs_miceFast =  model$vifs(posit_y,posit_x)
+rm(model)
+
+vifs_car
+vifs_miceFast
+
+m10 = microbenchmark::microbenchmark(car = { car::vif(lm(a~.,data=data.vifs))},
+                                    miceFast={
+                                      model = new(miceFast)
+                                      model$set_data(data_con)
+                                      vifs_miceFast =  model$vifs(posit_y,posit_x)
+                                      rm(model)
+
+                                    },
+                                    times=iters)
+m10
+
+g10 = autoplot(m10,log=FALSE)+theme_economist()+ ggtitle("vifs")
+
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g10.png",g10)
 
 #plot for README/Intro
 
@@ -481,11 +520,15 @@ data.frame(m8) %>% mutate(model = "linear regression noise - without grouping -
 data.frame(m9) %>% mutate(model = "linear regression bayes - without grouping - multiple 10")
 ))
 
-dats_plot = dats %>% group_by(model,expr) %>% summarise(mean_time_sec=mean(time/10**9)) %>% group_by(model) %>% mutate(relative_time=mean_time_sec/min(mean_time_sec))
+dats_plot = dats %>% group_by(model,expr) %>% summarise(mean_time_sec=mean(time/10**9)) %>%
+  group_by(model) %>%
+  mutate(relative_time=mean_time_sec/min(mean_time_sec)) %>%
+  rename('package'='expr')
 
-g_summary = ggplot(dats_plot,aes(model,mean_time_sec,fill=expr)) +
+g_summary = ggplot(dats_plot,aes(model,relative_time,fill=package)) +
   geom_bar(stat="identity",position="dodge") +
   theme(axis.text.x= element_text(angle=90)) +
-  ggtitle("Benchmarks - 10^6 obs (10% NA) 7 vars - 10^3 groups")
+  ggtitle(paste0("Benchmarks - 10^" ,power, "obs (20% NA) ",nr_var,"vars - 10^3 groups"))
+
+ggsave("C:/Users/user/Desktop/burk/own_R_packages/miceFast/inst/extdata/images/g_summary.png",g_summary)
 
-ggsave("C:/Users/user/Desktop/own_R_packages/miceFast/inst/extdata/images/g_summary.png",g_summary)
